"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Queue = void 0;

var _classPrivateFieldLooseBase2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldLooseBase"));

var _classPrivateFieldLooseKey2 = _interopRequireDefault(require("@babel/runtime/helpers/classPrivateFieldLooseKey"));

var _file2 = require("@skyekiwi/file");

var _util = require("@skyekiwi/util");

// Copyright 2021 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
var _file = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("file");

var _sealer = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("sealer");

var _calls = /*#__PURE__*/(0, _classPrivateFieldLooseKey2.default)("calls");

class Queue {
  constructor(file, sealer) {
    Object.defineProperty(this, _file, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _sealer, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _calls, {
      writable: true,
      value: void 0
    });
    (0, _classPrivateFieldLooseBase2.default)(this, _file)[_file] = file;
    (0, _classPrivateFieldLooseBase2.default)(this, _sealer)[_sealer] = sealer;
    (0, _classPrivateFieldLooseBase2.default)(this, _calls)[_calls] = [];
  }

  async init() {
    try {
      const logsRaw = await (0, _classPrivateFieldLooseBase2.default)(this, _file)[_file].readAll();
      const logs = logsRaw.split('|');

      for (const log of logs) {
        try {
          (0, _classPrivateFieldLooseBase2.default)(this, _calls)[_calls].push(this.decodeCall(log));
        } catch (err) {// pass
          // console.log(err)
        }
      }
    } catch (err) {// pass
      // the log is empty
    }
  }

  unlockSealer(key) {
    (0, _classPrivateFieldLooseBase2.default)(this, _sealer)[_sealer].unlock(key);
  }

  decodeCall(call) {
    let _call = call;
    let encrypted = false;

    if ((0, _util.isValidHex)(call)) {
      encrypted = true;
      _call = (0, _util.u8aToString)((0, _classPrivateFieldLooseBase2.default)(this, _sealer)[_sealer].decrypt((0, _util.hexToU8a)(call)));
    }

    const oneCall = _call.split('?');

    return {
      encrypted: encrypted,
      methodName: (0, _util.fromBase64)(oneCall[0]),
      origin: (0, _util.fromBase64)(oneCall[1]),
      parameters: (0, _util.fromBase64)(oneCall[2])
    };
  }

  encodeCall(call) {
    if (call.methodName.length < 0 || call.methodName.length > 32) {
      throw new Error('methodName must be between 0 - 32 bytes - calls/encodeCall');
    }

    if (call.parameters.length < 0 || call.parameters.length > 128) {
      throw new Error('parameters must be between 0 - 128 bytes - calls/encodeCall');
    }

    if (!(0, _util.isValidSubstrateAddress)(call.origin)) {
      throw new Error('origin must be a valid Substrate address - calls/encodeCall');
    }

    const callString = `${(0, _util.toBase64)(call.methodName)}?${(0, _util.toBase64)(call.origin)}?${(0, _util.toBase64)(call.parameters)}`;

    if (call.encrypted) {
      return (0, _util.u8aToHex)((0, _classPrivateFieldLooseBase2.default)(this, _sealer)[_sealer].encrypt((0, _util.stringToU8a)(callString), (0, _classPrivateFieldLooseBase2.default)(this, _sealer)[_sealer].getAuthorKey()));
    }

    return callString;
  }

  async writeCalls(path) {
    let currentCalls = '';

    for (const call of (0, _classPrivateFieldLooseBase2.default)(this, _calls)[_calls]) {
      currentCalls = currentCalls + this.encodeCall(call) + '|';
    }

    currentCalls = (0, _util.trimEnding)(currentCalls);
    await _file2.File.writeFile((0, _util.stringToU8a)(currentCalls), path, 'w');
  }

  parseAndPushCall(call) {
    if (!(0, _classPrivateFieldLooseBase2.default)(this, _calls)[_calls]) {
      throw new Error('must initialize first - s-contract/calls');
    }

    const _call = this.decodeCall(call);

    (0, _classPrivateFieldLooseBase2.default)(this, _calls)[_calls].push(_call);
  }

  pushCall(call) {
    (0, _classPrivateFieldLooseBase2.default)(this, _calls)[_calls].push(call);
  }

  getCalls() {
    return (0, _classPrivateFieldLooseBase2.default)(this, _calls)[_calls];
  }

}

exports.Queue = Queue;