import _classPrivateFieldLooseBase from "@babel/runtime/helpers/esm/classPrivateFieldLooseBase";
import _classPrivateFieldLooseKey from "@babel/runtime/helpers/esm/classPrivateFieldLooseKey";
// Copyright 2021 @skyekiwi/s-contract authors & contributors
// SPDX-License-Identifier: Apache-2.0
import { File } from '@skyekiwi/file';
import { fromBase64, hexToU8a, isValidHex, isValidSubstrateAddress, stringToU8a, toBase64, trimEnding, u8aToHex, u8aToString } from '@skyekiwi/util';

var _file = /*#__PURE__*/_classPrivateFieldLooseKey("file");

var _sealer = /*#__PURE__*/_classPrivateFieldLooseKey("sealer");

var _calls = /*#__PURE__*/_classPrivateFieldLooseKey("calls");

export class Queue {
  constructor(file, sealer) {
    Object.defineProperty(this, _file, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _sealer, {
      writable: true,
      value: void 0
    });
    Object.defineProperty(this, _calls, {
      writable: true,
      value: void 0
    });
    _classPrivateFieldLooseBase(this, _file)[_file] = file;
    _classPrivateFieldLooseBase(this, _sealer)[_sealer] = sealer;
    _classPrivateFieldLooseBase(this, _calls)[_calls] = [];
  }

  async init() {
    try {
      const logsRaw = await _classPrivateFieldLooseBase(this, _file)[_file].readAll();
      const logs = logsRaw.split('|');

      for (const log of logs) {
        try {
          _classPrivateFieldLooseBase(this, _calls)[_calls].push(this.decodeCall(log));
        } catch (err) {// pass
          // console.log(err)
        }
      }
    } catch (err) {// pass
      // the log is empty
    }
  }

  unlockSealer(key) {
    _classPrivateFieldLooseBase(this, _sealer)[_sealer].unlock(key);
  }

  decodeCall(call) {
    let _call = call;
    let encrypted = false;

    if (isValidHex(call)) {
      encrypted = true;
      _call = u8aToString(_classPrivateFieldLooseBase(this, _sealer)[_sealer].decrypt(hexToU8a(call)));
    }

    const oneCall = _call.split('?');

    return {
      encrypted: encrypted,
      methodName: fromBase64(oneCall[0]),
      origin: fromBase64(oneCall[1]),
      parameters: fromBase64(oneCall[2])
    };
  }

  encodeCall(call) {
    if (call.methodName.length < 0 || call.methodName.length > 32) {
      throw new Error('methodName must be between 0 - 32 bytes - calls/encodeCall');
    }

    if (call.parameters.length < 0 || call.parameters.length > 128) {
      throw new Error('parameters must be between 0 - 128 bytes - calls/encodeCall');
    }

    if (!isValidSubstrateAddress(call.origin)) {
      throw new Error('origin must be a valid Substrate address - calls/encodeCall');
    }

    const callString = `${toBase64(call.methodName)}?${toBase64(call.origin)}?${toBase64(call.parameters)}`;

    if (call.encrypted) {
      return u8aToHex(_classPrivateFieldLooseBase(this, _sealer)[_sealer].encrypt(stringToU8a(callString), _classPrivateFieldLooseBase(this, _sealer)[_sealer].getAuthorKey()));
    }

    return callString;
  }

  async writeCalls(path) {
    let currentCalls = '';

    for (const call of _classPrivateFieldLooseBase(this, _calls)[_calls]) {
      currentCalls = currentCalls + this.encodeCall(call) + '|';
    }

    currentCalls = trimEnding(currentCalls);
    await File.writeFile(stringToU8a(currentCalls), path, 'w');
  }

  parseAndPushCall(call) {
    if (!_classPrivateFieldLooseBase(this, _calls)[_calls]) {
      throw new Error('must initialize first - s-contract/calls');
    }

    const _call = this.decodeCall(call);

    _classPrivateFieldLooseBase(this, _calls)[_calls].push(_call);
  }

  pushCall(call) {
    _classPrivateFieldLooseBase(this, _calls)[_calls].push(call);
  }

  getCalls() {
    return _classPrivateFieldLooseBase(this, _calls)[_calls];
  }

}